float Material_U;

float4x4 MatrixW;
float4x4 MatrixWVP;

float4 MaterialAmbientColor;
float4 MaterialDiffuseColor;

float4 LightDiffuseColor;
float4 LightAmbientColor;
float3 LightDirection;

float3 EyeDirection;

texture MeshTexture;
texture ShadeTexture;

sampler MeshTextureSampler = 
sampler_state{
	Texture = < MeshTexture >;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

sampler ShadeTextureSampler = 
sampler_state{
	Texture = < ShadeTexture >;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

// _VF[_

// gD[VF[_
struct VS_OUTPUT_Shade{
	float4 Pos			: POSITION;
	float4 Col			: COLOR0;
	float2 TexUV		: TEXCOORD0;
	float2 ShadeTexUV	: TEXCOORD1;
};

VS_OUTPUT_Shade VS_Shade(
	float4 Pos			: POSITION,
	float3 Normal		: NORMAL,
	float2 TexCoord0	: TEXCOORD0 )
{
	VS_OUTPUT_Shade Out = (VS_OUTPUT_Shade)0;
	Out.Pos = mul( Pos, MatrixWVP );

	float3 n = mul( Normal, (float3x3)MatrixW );
	Out.ShadeTexUV.x = max( dot( n, -LightDirection ), 0 );
	Out.ShadeTexUV.y = Material_U;

	// XyL
	float3 Eye = normalize( EyeDirection - mul( Pos, MatrixW ) );
	float3 r = -Eye + 2.0f * dot( n, Eye ) * n;
	Out.Col = pow( max( 0,dot( -LightDirection, r ) ), 4 );

	// Œ蒸_̒_FvZ
//	float3 NormalW = normalize( mul( Normal, (float3x3)MatrixW ) );
//	Out.Col = MaterialDiffuseColor * LightDiffuseColor * max( dot( -LightDirection, NormalW ), 0 ) + MaterialAmbientColor * LightAmbientColor;

	Out.TexUV = TexCoord0;

	return Out;
}

// ֊sVF[_
float4 VS_Outline(
      float4 Pos		: POSITION,
      float3 Normal		: NORMAL,
      float2 TexCoord0	: TEXCOORD0 ) : POSITION
{
	float4 a = mul( Pos, MatrixWVP );
	float4 b = mul( Normal, MatrixWVP );
	b.zw = 0;
	b = normalize( b ) * 0.004; // ̑
	return b * a.w + a;
}

// sNZVF[_
float4 PS_Shade( VS_OUTPUT_Shade In ) : COLOR0
{
	return tex2D( MeshTextureSampler, In.TexUV ) * tex2D( ShadeTextureSampler, In.ShadeTexUV ) * ( 1 + In.Col / 2 );
}

// eNjbN
technique ShadeShader{
	pass P0{
		VertexShader = compile vs_2_0 VS_Shade();
		PixelShader  = compile ps_2_0 PS_Shade();
	}
}

technique OutlineShader{
    pass P0{
		VertexShader = compile vs_2_0 VS_Outline();
	}
}
